{ --------------------------------------------------------------------------- }
{ CPUTYPE.PAS   Turbo Pascal TMi0SDGL 2 interface unit.         Version 2.07  }
{                                                                             }
{ Too-Much-in-0ne-So-Don't-Get-Lost(tm) Revision 2 CPU/FPU detection library. }
{ Copyright(c) 1996-98 by B-coolWare.  Written by Bobby Z.                    }
{ --------------------------------------------------------------------------- }

{$A+,I-,S-,X+}
{$IFDEF Win32}
{$H-}
{$ENDIF}
(*
{$DEFINE NormalizeFreq} { use frequency normalization tables }
*)

unit CPUType;

interface

type
        cpuid1Layout = record
                        Extra : Byte;
                        Family: Byte;
                        Model : Byte;
                        Step  : Byte;
                       end;

        customCpuid = record
                        eax,
                        ebx,
                        ecx,
                        edx   : {$IFDEF WIN32} Integer {$ELSE} LongInt {$ENDIF};
                      end;

const
        libVersion  = $0207;

        cpu         : Byte = $FF;
        fpu         : Byte = $FF;
        extFlags    : Word = 0;
        cpuid0      : array[0..11] of Char = #0#0#0#0#0#0#0#0#0#0#0#0;
        cpuid1      : {$IFDEF WIN32} Integer {$ELSE} LongInt {$ENDIF}= 0;
        cpuFeatures : {$IFDEF WIN32} Integer {$ELSE} LongInt {$ENDIF}= 0;

{ CPU type constants }

        i8088           = 00;
        i8086           = 01;
        i80C88          = 02;
        i80C86          = 03;
        i80188          = 04;
        i80186          = 05;
        necV20          = 06;
        necV30          = 07;
        i80286          = 08;
        i80386sx        = 09;
        i80386dx        = 10;
        i386sl          = 11;
        ibm386slc       = 12;
        am386sx         = 13;
        am386dx         = 14;
        ct38600         = 15;
        ct38600SX       = 16;
        RapidCAD        = 17;
        i486sx          = 18;
        i486dx          = 19;
        ibm486slc       = 20;
        ibm486slc2      = 21;
        ibm486bl3       = 22;
        Cx486           = 23;
        umcU5S          = 24;
        umcU5D          = 25;
        am486           = 26;
        iPentium        = 27;
        iP54C           = 28;
        CxM1            = 29;
        amdK5           = 30;
        Nx586           = 31;
        iPentiumPro     = 32;
        amdK6           = 33;
        iP7             = 34;
        iP8             = 35;
        CxM2            = 36;
        Am486DX         = 37;
        amdK5_2         = 38;
        WinChipC6       = 39;
        i486sl          = 40;

{ FPU type constants }

        fpuInternal     = 100;
        fpuNone         = 0;
        i8087           = 1;
        i80287          = 2;
        i80287xl        = 3;
        i80387          = 4;
        rCAD            = 5;
        cx287           = 6;
        cx387           = 7;
        cx487           = 8;
        cxEMC87         = 9;
        iit287          = 10;
        iit387          = 11;
        iit487          = 12;
        ct387           = 13;
        ulsi387         = 14;
        ulsi487         = 15;
        i487sx          = 16;
        Nx587           = 17;
        iit487DLC       = 18;
        i387SLMobile    = 19;

{ misc constants }

        efHasFPUonChip  = $0001;
        efWeitekPresent = $0002;
        efCPUIDSupport  = $0004;
        efDXType        = efCPUIDSupport+efHasFPUOnChip;
        efEmulatedFPU   = $0008;
        efCentaurLevel  = $0010;


function getVersion : Word; {$IFDEF _DLL}export;{$ENDIF}

function cpu_Type : {$IFDEF Win32} ShortString;{$ELSE} String; {$ENDIF} { returns CPU name }

function fpu_Type : {$IFDEF Win32} ShortString;{$ELSE} String; {$ENDIF} { returns FPU name }

function cpu_Speed : Word;  { returns CPU clock in MHz }

function fcpu_Speed : Real; { returns floating point CPU clock freq }

{$IFNDEF Win32}
function getCacheSize : Word; far; { returns L1 cache size in Kb }
{$ENDIF}

procedure CxCPUIDEnable; {$IFDEF Windows} export; {$ELSE}
                         {$IFDEF Win32} export; {$ELSE} far; {$ENDIF}{$ENDIF}

{$IFNDEF Win32}
procedure getCPUID( Level : LongInt;
                    Result : Pointer ); {$IFDEF Windows} export; {$ELSE} far;
                                        {$ENDIF}
{$ENDIF}

{$IFNDEF Win32}
{$IFNDEF Ver80}
{$IFDEF Ver70}
{$IFNDEF DPMI}
 {$IFNDEF Windows}
function isV86 : Boolean; far; { this routine is useful for real mode only }
 {$ENDIF} { Windows }
{$ENDIF} { DPMI }
{$ELSE} { not Ver70, no protected mode or windows }
function isV86 : Boolean; far;
{$ENDIF} { Ver70 }
{$ENDIF} { Ver80 }
{$ENDIF} { Win32 }
implementation

{ do not change following constants! }
const
        fpuDenormal   : array [0..9] of Byte = (1,0,0,0,0,0,0,0,0,0);
        fpuOp1        : array [0..9] of Byte = ($F0,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$3F);
        fpu_53bit_Prec: word = $02F7;
        speedShift    : word = 0;
        speedModifier : word = 1;
        speedTable    : array[i8088..i486sl] of {$IFDEF Win32} Integer {$ELSE} LongInt {$ENDIF} =
                        (
                         $0002AD26, { i8088 }
                         $0002AD26, { i8086 }
                         $0002AD26, { i80C88 }
                         $0002AD26, { i80C86 }
                         $0000BA6F, { i80188 }
                         $0000BA6F, { i80186 }
                         $00008C38, { necV20 }
                         $0000912C, { necV30 }
                         $00006FDC, { i80286 }
                         $00007480, { i80386SX }
                         $00007480, { i80386DX }
                         $00007480, { i386sl }
                         $00007480, { ibm386slc }
                         $00007415, { Am386SX }
                         $00007415, { Am386DX }
                         $00007480, { CT38600 }
                         $00007480, { CT38600SX }
                         $00007415, { RapidCAD }
                         $00007480, { i486SX }
                         $00007480, { i486DX }
                         $00007486, { ibm486slc }
                         $00007486, { ibm486slc2 }
                         $00007486, { ibm486bl3 }
                         $0000668A, { Cx486 }
                         $00003C90, { umcU5S }
                         $00003C90, { umcU5D }
                         $00007480, { Am486 }
                         $00007900, { Pentium }
                         $00007950, { P54C }
                         $00004800, { CxM1 }
                         $000061D0, { amdK5 }
                         $0000792E, { Nx586       !!! needs adjustment }
                         $00006BCD, { iPentiumPro }
                         $00003CD2, { amdK6 }
                         $00003079, { P7          !!! needs adjustment }
                         $00003079, { P8          !!! needs adjustment }
                         $0000D688, { CxM2        !!! needs adjustment }
                         $00007480, { Am486DX }
                         $00008AC6, { AMD K5 models 2 and 3 }
                         $00003900, { WinChip C6  !!! needs adjustment }
                         $00007480  { i486SL }
                        );

end.
